<?php

class FilesHandler extends request_handler {

    const int SEARCH_RESULTS_PER_PAGE = 50;
    const int SEARCH_MIN_QUERY_LENGTH = 3;

    public function GET_files() {
        $collections = array_map(fn(FilesCollection $c) => new CollectionItem($c), FilesCollection::cases());
        $books = files::books_get();
        $misc = files::books_get(category: BookCategory::MISC);
        $this->skin->addMeta([
            '@title' => '$meta_files_title',
            '@description' => '$meta_files_description'
        ]);
        $this->skin->setTitle('$files');
        $this->skin->setRenderOptions(['head_section' => 'files']);
        $this->skin->renderPage('files_index.twig', [
            'collections' => $collections,
            'books' => $books,
            'misc' => $misc
        ]);
    }

    public function GET_folder() {
        list($folder_id) = $this->input('i:folder_id');

        $parents = files::books_get_folder($folder_id, true);
        if (!$parents)
            self::notFound();

        if (count($parents) > 1)
            $parents = array_reverse($parents);

        $folder = $parents[count($parents)-1];
        $files = files::books_get($folder_id, category: $folder->category);

        $bc = [
            ['text' => lang('files'), 'url' => '/files/'],
        ];
        if ($parents) {
            for ($i = 0; $i < count($parents)-1; $i++) {
                $parent = $parents[$i];
                $bc_item = ['text' => $parent->getTitle()];
                if ($i < count($parents)-1)
                    $bc_item['url'] = $parent->getUrl();
                $bc[] = $bc_item;
            }
        }
        $bc[] = ['text' => $folder->title];

        $this->skin->addMeta([
            '@title' => lang('meta_files_book_folder_title', $folder->getTitle()),
            '@description' => lang('meta_files_book_folder_description', $folder->getTitle())
        ]);
        $this->skin->setTitle(lang('files').' - '.$folder->title);
        $this->skin->renderPage('files_folder.twig', [
            'folder' => $folder,
            'bc' => $bc,
            'files' => $files
        ]);
    }

    public function GET_collection() {
        list($collection, $folder_id, $query, $offset) = $this->input('collection, i:folder_id, q, i:offset');
        $collection = FilesCollection::from($collection);
        $parents = null;

        $query = trim($query);
        if (!$query)
            $query = null;

        $this->skin->exportStrings('/^files_(.*?)_collection$/');
        $this->skin->exportStrings([
            'files_search_results_count'
        ]);

        $vars = [];
        $text_excerpts = null;
        $func_prefix = $collection->value;

        if ($query !== null) {
            $files = call_user_func("files::{$func_prefix}_search", $query, $offset, self::SEARCH_RESULTS_PER_PAGE);
            $vars += [
                'search_count' => $files['count'],
                'search_query' => $query
            ];

            /** @var WFFCollectionItem[]|MDFCollectionItem[]|BaconianaCollectionItem[] $files */
            $files = $files['items'];

            $query_words = array_map('mb_strtolower', preg_split('/\s+/', $query));
            $found = [];
            $result_ids = [];
            foreach ($files as $file) {
                if ($file->isFolder())
                    continue;
                $result_ids[] = $file->id;

                switch ($collection) {
                    case FilesCollection::MercureDeFrance:
                        $candidates = [
                            $file->date,
                            (string)$file->issue
                        ];
                        break;
                    case FilesCollection::WilliamFriedman:
                        $candidates = [
                            mb_strtolower($file->getTitle()),
                            strtolower($file->documentId)
                        ];
                        break;
                    case FilesCollection::Baconiana:
                        $candidates = [
                            // TODO
                        ];
                        break;
                }

                foreach ($candidates as $haystack) {
                    foreach ($query_words as $qw) {
                        if (mb_strpos($haystack, $qw) !== false) {
                            $found[$file->id] = true;
                            continue 2;
                        }
                    }
                }
            }

            $found = array_map('intval', array_keys($found));
            $not_found = array_diff($result_ids, $found);
            if (!empty($not_found))
                $text_excerpts = call_user_func("files::{$func_prefix}_get_text_excerpts", $not_found, $query_words);

            if (self::isXhrRequest()) {
                self::ajaxOk([
                    ...$vars,
                    'new_offset' => $offset + count($files),
                    'html' => skin::getInstance()->render('files_list.twig', [
                        'files' => $files,
                        'query' => $query,
                        'text_excerpts' => $text_excerpts
                    ])
                ]);
            }
        } else {
            if (in_array($collection, [FilesCollection::WilliamFriedman, FilesCollection::Baconiana]) && $folder_id) {
                $parents = call_user_func("files::{$func_prefix}_get_folder", $folder_id, true);
                if (!$parents)
                    self::notFound();
                if (count($parents) > 1)
                    $parents = array_reverse($parents);
            }
            $files = call_user_func("files::{$func_prefix}_get", $folder_id);
        }

        $title = lang('files_'.$collection->value.'_collection');
        if ($folder_id && $parents)
            $title .= ' - '.htmlescape($parents[count($parents)-1]->getTitle());
        if ($query)
            $title .= ' - '.htmlescape($query);
        $this->skin->setTitle($title);

        if (!$folder_id && !$query) {
            $this->skin->addMeta([
                '@title' => lang('4in1').' - '.lang('meta_files_collection_title', lang('files_'.$collection->value.'_collection')),
                '@description' => lang('meta_files_'.$collection->value.'_description')
            ]);
        } else if ($query || $parents) {
            $this->skin->addMeta([
                '@title' => lang('4in1').' - '.$title,
                '@description' => lang('meta_files_'.($query ? 'search' : 'folder').'_description',
                    $query ?: $parents[count($parents)-1]->getTitle(),
                    lang('files_'.$collection->value.'_collection'))
            ]);
        }

        $bc = [
            ['text' => lang('files'), 'url' => '/files/'],
        ];
        if ($parents) {
            $bc[] = ['text' => lang('files_'.$collection->value.'_collection_short'), 'url' => "/files/{$collection->value}/"];
            for ($i = 0; $i < count($parents); $i++) {
                $parent = $parents[$i];
                $bc_item = ['text' => $parent->getTitle()];
                if ($i < count($parents)-1)
                    $bc_item['url'] = $parent->getUrl();
                $bc[] = $bc_item;
            }
        } else {
            $bc[] = ['text' => lang('files_'.$collection->value.'_collection')];
        }

        $js_params = [
            'container' => 'files_list',
            'per_page' => self::SEARCH_RESULTS_PER_PAGE,
            'min_query_length' => self::SEARCH_MIN_QUERY_LENGTH,
            'base_url' => "/files/{$collection->value}/",
            'query' => $vars['search_query'],
            'count' => $vars['search_count'],
            'collection_name' => $collection->value,
            'inited_with_search' => !!($vars['search_query'] ?? "")
        ];

        $this->skin->set($vars);
        $this->skin->set([
            'collection' => $collection->value,
            'files' => $files,
            'bc' => $bc,
            'do_show_search' => empty($parents),
            'do_show_more' => $vars['search_count'] > 0 && count($files) < $vars['search_count'],
            // 'search_results_per_page' => self::SEARCH_RESULTS_PER_PAGE,
            // 'search_min_query_length' => self::SEARCH_MIN_QUERY_LENGTH,
            'text_excerpts' => $text_excerpts,
            'js_params' => $js_params,
        ]);
        $this->skin->renderPage('files_collection.twig');
    }

}